function [sys,x0,str,ts] = buffer_1(t,x,u,flag)
%  This S-function stores the past four values of the input in 
%  a buffer and outputs the sum of those values.  It 
%  demonstrates how discrete states can be used as memory 
%  elements that store specified 4 values.

% The following outlines the general structure of an S-function.
%

switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0,      % 0 ==> return sizes
    [sys,x0,str,ts]=mdlInitializeSizes;   

  %%%%%%%%%%
  % Update %
  %%%%%%%%%%
  case 2,      % 2 ==> return next discrete state
    sys=mdlUpdate(t,x,u);

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,      % 3 ==> return outputs
    sys=mdlOutputs(t,x,u);
    
  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case 9,      %  9 ==> Termination
    sys=mdlTerminate(t,x,u);

 
  %%%%%%%%%%%%%%%%%%%%
  % Unexpected flags %
  %%%%%%%%%%%%%%%%%%%%
  otherwise    % all other flags 

    error(['Unhandled flag = ',num2str(flag)]);

end

% end buffer_1

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes
%
sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = 4;  %  4 discrete states to hold the "old" 
                           %  values
sizes.NumOutputs     = 1;
sizes.NumInputs      = 1;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 0;  % at least one sample time is needed

sys = simsizes(sizes);

%
% initialize the initial conditions
%
x0  = zeros(1,4);          

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [];

% end mdlInitializeSizes
%

%
%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,x,u)

sys = zeros(4,1);    % initialize sys
sys(4) = x(3);       % here we are setting the values for
sys(3) = x(2);       % x(4), x(3), x(2), and x(1) respectively, for
sys(2) = x(1);       % the NEXT integration step
sys(1) = u;

% end mdlUpdate

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

sys = sum(x);  % output the sum of the past four values

% end mdlOutputs

%
%=============================================================================
% mdlTerminate
% Perform any end of simulation tasks.
%=============================================================================
%
function sys=mdlTerminate(t,x,u)

sys = [];        % set []

% end mdlTerminate
